#! /usr/bin/perl
#
# $Header: //sapdb/TOOLSRC/develop/sys/src/BuildPackage/Vars.pm#2 $
# $DateTime: 2002/02/26 13:42:33 $
# $Change: 17115 $
#
# Desc:	- subs to handle archive tree LC_POOL 
#		
#
		
package BuildPackage::LCPool;

use BuildPackage::Vars;
use BuildPackage::Tools;
use Cwd;
use Exporter;

@ISA = ('Exporter');

@EXPORT = ('findFreePoolSlot','linkSlot','copyItems');


#
# initPackage() set global package vars from outside - especially needed for err msg handler 
#

sub initPackage{
	my (%args) = @_;
	foreach my $sym (keys(%args)){
		*{$sym} = $args{$sym};
	}
	return 1;
}


#
# findFreePoolSlot():
#	- searches in a pool directory $pool_root free (without $testfile) subdirs  
# 	- next free is one above latest/highest valid (slot with $testfile)		
#     - $digit_num is number of digits for slot directories (3 means from '000' up to '999')
#	- return full path of next free slot as scalar
#

sub findFreePoolSlot{
	my ($pool_root,$testfile,$digit_num) = @_;
	defined $pool_root and defined $testfile or return undef;
	defined $digit_num and $digit_num =~ /^\d+$/ and $digit_num > 0 or $digit_num = 3; # default length
	my $new_dir;
	my $num_str;
	if(-d $pool_root){
		opendir(DH,$pool_root) or err("FreePoolSlot(): cannot open dir $pool_root\n");
		my @pool = sort grep {/^\d{$digit_num}$/} readdir(DH);
		closedir(DH);
		if($#pool == -1){
			$num_str = '0' x $digit_num;
		}
		else{
			while(1){
				my $top = pop @pool;			
				if(-f "$pool_root/$top/$testfile"){
					my $num = $top + 1;
					$num_str = 0 x ($digit_num - length($num)) . $num;			
					last;
				}
				if($#pool == -1){
					$num_str = '0' x $digit_num;
					last;
				}		
			}
		}
	}
	else{
		$num_str = '0' x $digit_num;
	}
	return "$pool_root/$num_str";
}


#
# copyItems() copy files and fs trees with original owner/group, timestamp and access rights 
# items = files/directories 
#

sub copyItems{
	my ($src_tree,$dest_tree,$item_list) = @_;
	-d $src_tree or return undef;
	ref($item_list) eq 'ARRAY' and $#{@$item_list} > -1 or return undef;
	my @not_found = ();
	foreach (@$item_list){
		-e "$src_tree/$_" or push @not_found,$_;
	}
	unless ($#not_found == -1){
		err("items:\n\t".
		    join("\n\t",@not_found)."\n".
		    "in $src_tree not found\n");
	}	

	-d $dest_tree or (makedir($dest_tree,0775) or err("cannot create directory \"$dest_tree\"\n"));
	my $tar = ($^O =~ /mswin/i) ? 'tar' : 'gtar';	
	do_cmd("$tar -C $src_tree -cpO ".join(' ',@$item_list)." | $tar -xpf- -C $dest_tree");
	return 1;
}


#
# linkSlot(): create a symblolic link $link_name in $pool_root to given slot
#

sub linkSlot{
	my ($slot_path,$link_name) = @_;
	-d $slot_path or err("linkSlot(): directory $slot_path not found\n");
	my ($pool_root,$slot) = ($slot_path =~ /^(.+)\/(\d+)\/{0,1}$/);
	my $cwd = getcwd();
	chdir($pool_root) || err("linkSlot(): cannot change into directory $pool_root: $!\n");
	if(-e $link_name){
		unless( -l $link_name){
			err("linkSlot(): $pool_root/$link_name exists and is no link\n");		
		}
		unlink($link_name) or err("linkSlot(): cannot remove symlink $pool_root/$link_name: $!\n"); 
	}
	symlink($slot,$link_name) or err("linkSlot(): cannot create link \"$link_name\" to slot $slot\n"); 
	chdir($cwd) or err("linkSlot(): cannot change into directory $cwd: $!\n");
	return 1;
}

1;

