
import string

class CommandCollection:
    def __init__ (self, session, stringDict):
        self.session = session
        self.stringDict = stringDict

    def getStringForCommand (self, commandName):
        return self.stringDict [commandName]

    def __getattr__ (self, name):
        try:
            cmd = self.getStringForCommand (name)
        except KeyError:
            raise AttributeError (name)
        prepared = self.session.prepare (cmd)
        callable = prepared.execute
        setattr (self, name, callable)
        return callable

class ExtendedConnection:
    def __init__ (self, connection):
        self.connection = connection
        self.stmtCache = {}

    def sqlX (self, cmd, args):
        try:
            prepared = self.stmtCache [cmd]
        except KeyError:
            prepared = self.connection.prepare (cmd)
            self.stmtCache [cmd] = prepared
        return prepared.execute (args)

    def __getattr__ (self, key):
        return getattr (self.connection, key)

def connect (options, userInfoName = 'userInfo',
        dbnameName = 'dbname', nodeName = 'node',
        propName = 'connectProperties', module = None):
    userInfo = getattr (options, userInfoName)
    dbname = getattr (options, dbnameName)
    node = getattr (options, nodeName)
    try:
        properties = getattr (options, propName)
    except AttributeError:
        properties = ''
    pos = string.find (userInfo, ',')
    if pos != -1:
        username = userInfo [:pos]
        pwd = userInfo [pos + 1:]
    else:
        username = userInfo
        pwd = ''
    if not node:
        pos = string.find (dbname, ':')
        if pos != -1:
            node = dbname [:pos]
            dbname = dbname [pos + 1:]
        else:
            node = ''
    if module == None:
        import sapdb
        module = sapdb
    result = module.connect (username, pwd, dbname, node, properties)
    return result

stddbOptions = [
    ('u', 'userInfo', ':', None, 'user <name>,<pwd>'),
    ('d', 'dbname', ':', None, 'db name'),
    ('n', 'node', ':', '', 'host name'),
    ('P', 'connectProperties', '::', None, 'connect properties'),
    ]


