
import sys
import os
import string
import regex

independentVersion = ' '

class VmakeItem:
    name = '<unknown>'
    version = independentVersion

    def __init__ (self, kind):
        self.kind = kind

    def __repr__ (self):
        return "<VMAKE %s '%s'>" % (self.kind, self.name)

    def asKey (self):
        return (self.name, self.version)

class VmakeParser:
    def __init__ (self):
        self.handler = {}
        self.unknownHandler = unknownHandler

    def registerHandler (self, kind, handler):
        if kind == None:
            self.unknownHandler = handler
        else:
            self.handler [kind] = handler

    def callHandler (self, kind, item):
        try:
            handler = self.handler [kind]
        except KeyError:
            handler = self.unknownHandler
        handler (item)

    def parseCollection (self, target, speed = 's'):
        if type (target) == type ([]):
            target = string.join (target)
        instream = PipeReader ('perl -S im' + speed + '.pl -c ' + target)
        try:
            while 1:
                self.parseItem (instream)
        except EOFError:
            pass

    simplePropertyRE = regex.compile (' +\([a-z_]+\) +\([^ ]+\)')
    nameRE = regex.compile (' +\([a-z]+\)')
    valueRE = regex.compile (" +'\([^']*\)'[ \t]+'\([^']*\)'")
    optionsRE = regex.compile (" +'\([^']*\)'")

    def parseItem (self, instream):
        line = instream.skipEmptyLines ()
        kind = string.strip (line)
        item = VmakeItem (kind)
        line = instream.readline ()
        simplePropertyRE = self.simplePropertyRE
        nameRE = self.nameRE
        valueRE = self.valueRE
        optionsRE = self.optionsRE
        while simplePropertyRE.match (line) != -1:
            propName, propValue = simplePropertyRE.group (1, 2)
            if propValue [0] == "'":
                propValue = propValue [1:-1]
            elif propValue [0] in "0123456789":
                propValue = int (propValue)
            setattr (item, propName, propValue)
            line = instream.readline ()
        inProperties = 1
        while inProperties:
            if nameRE.match (line) != -1:
                currentList = []
                setattr (item, nameRE.group (1), currentList)
            elif valueRE.match (line) != -1:
                currentList.append (valueRE.group (1, 2))
            elif optionsRE.match (line) != -1:
                currentList.append (optionsRE.group (1))
            else:
                instream.pushbackLine (line)
                inProperties = None
                break
            line = instream.readline ()
        # self.findItemBreak (instream, line)
        self.callHandler (kind, item)
        return item

    def findItemBreak (self, instream, line):
        while 1:
            if not line:
                # empty lines as delimiter
                break
            if line [0] != ' ':
                # new items start at first column
                instream.pushbackLine (line)
                break
            line = instream.readline ()

class PipeReader:
    def __init__ (self, command):
        self.instream = os.popen (command, 'r')
        self.currentLine = None

    def readline (self):
        if self.currentLine != None:
            result = self.currentLine
            self.currentLine = None
        else:
            result = self.instream.readline ()
            if not result:
                raise EOFError
            result = result [:-1]   # strip \n
        return result

    def pushbackLine (self, line):
        self.currentLine = line

    def skipEmptyLines (self):
        while 1:
            line = self.readline ()
            if line:
                return line


def unknownHandler (item):
    sys.stderr.write ('no handler defined for ' + `item` + '\n')

def ignoreHandler (item):
    pass

